//=============================================================================
//
// file :        PLCaccess.h
//
// description : Include for the PLCaccess class.
//
// project :	access control PLC manager
//
// $Author: graziano $
//
// $Revision: 1.27 $
//
// $Log: PLCaccess.h,v $
// Revision 1.27  2017-09-07 09:47:03  graziano
// fixed bug in disabled_boards and resettable_boards size
//
// Revision 1.26  2017-08-21 12:54:37  graziano
// added TopUp_GeV, TopUp_Min_I and booster_board_disabled_xxx attributes
//
// Revision 1.25  2017-08-11 14:59:27  graziano
// added log of db51 with sr and fermi pss format,
// fixed timestamp
//
// Revision 1.24  2017-03-29 14:36:50  claudio
// ticket 1934: compile for tango-9
//
// Revision 1.23  2016-04-06 12:08:23  graziano
// fixed bug in array sizes
//
// Revision 1.22  2016-04-06 11:54:39  graziano
// fixed dst
//
// Revision 1.21  2016-01-15 11:21:02  graziano
// added db state and error
//
// Revision 1.20  2014-11-13 09:59:56  graziano
// tango 8.1.2.c and Device_4Impl
//
// Revision 1.19  2013-01-04 10:38:20  graziano
// fixed log of people
//
// Revision 1.18  2010-05-17 08:08:16  zambonl
// ndd
//
// Revision 1.17  2010/01/21 12:27:32  zambonl
// ndd
//
// Revision 1.16  2010/01/12 09:20:37  zambonl
// ndd
//
// Revision 1.15  2009/11/27 15:07:14  sara
// fixed bug in plcsrvthread, wrong msec in timestamp
//
// Revision 1.14  2009/11/25 15:41:45  sara
// added reset topup error command
//
// Revision 1.13  2009/11/18 16:53:36  sara
// topup version
//
// Revision 1.12  2009/05/11 15:52:39  sara
// fixes problems with long packet reassembly
//
// Revision 1.11  2008/10/13 08:32:02  sara
// safe message recognition, adjustments in ctrl_names and ronda_names
// attributes
//
// Revision 1.10  2008/07/01 14:11:17  sara
// *** empty log message ***
//
// Revision 1.9  2008/07/01 13:39:26  sara
// added PLC clock syncronizing command
//
// Revision 1.8  2008/03/17 14:29:56  sara
// Includes access logging and a new attribute: Ramping_condition
//
// Revision 1.7  2007/11/22 16:49:14  sara
// new attributes:Gun_trigger_on, Gun_trigger_interlock_abi, Gun_trigger_access_abi
// enabled database log
//
// Revision 1.6  2007/11/05 10:37:33  sara
// added commands gun_on and gun_off, modified and added signals
//
// Revision 1.5  2007/10/08 14:40:21  sara
// changes in ronda booster fail cause
//
// Revision 1.4  2007/10/05 09:14:36  sara
// added attribute "version"
//
// Revision 1.3  2007/08/13 10:26:53  sara
// timing thread for receive from PLC timeout
//
// Revision 1.2  2007/07/02 09:32:44  sara
// Modifica di prova. Aggiunta una riga di commento.
//
// Revision 1.1.1.1  2007/07/02 09:29:34  sara
// Initial release
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza, Trieste ITALY
//
//=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================
//

#ifndef _PLCACCESS_H
#define _PLCACCESS_H

#include <tango.h>
#include <sys/time.h>
#include <time.h>

#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <deque>
#include <sys/sem.h>

#include <mysql.h>
//using namespace Tango;

/**
 * @author	$Author: graziano $
 * @version	$Revision: 1.27 $
 */

 //	Add your own constants definitions here.
 //-----------------------------------------------

//size of the message buffer
#define	BUF_LEN	2000
//size of the message buffer
//#define	BUF_WR	100
//timestamp size in bytes
#define	TSTAMP_SIZE	8
//badge string size in words
#define	BADGE_SIZE	16
//number of places for testimoni
#define	NUM_POSTO	8
//number of bit
#define	NUM_BIT		16
//db names length
#define	DB_NAMES_LEN	116
//max size of the db query
#define	DB_QUERY_LEN	2048


//TOPUP CONST
#define	DB181_BYTE_LEN	50	//was 146, verify!!!
#define	DB181_BIT_LEN	8*DB181_BYTE_LEN	//400, was 1168
#define	TOPUP_BLOCK_LEN	9
#define	TOPUP_BUF_LEN	3000	//size of the message buffer
#define	CURR_BUF_XLEN	200	
//TOPUP CONST


//command codes
#define	ECHO_50_CMD	50	//request db50 command - badge and posti
#define	ECHO_51_CMD	51	//request db51 command - all states
#define	DEL_BADGE_CMD	101	//delete badge command
#define	BST_CMD		102	//beam stopper,top-up,reint pasv, ack eme
#define	RONDA_CMD	103	//req,start,end ronda; booster and inj doors
#define	ECHO_121_CMD	121	//request db121 command - ronda enabled people 
#define	ECHO_122_CMD	122	//request db122 command - controlled access enabled people
#define	SYNC_CMD	7777	//clock alignment cmd

//command codes
#define	CMD_REQ	6	//request a command
#define	CMD_OK	9	//confirmation command
#define TIME_CONFIRM	1500000	//time in usec between command and confirm
#define MESG_LOST_CONNECTION	"Persa la connessione con PLC"
#define MESG_PLC_TIMEOUT	"Scaduto timeout risposta da PLC"

//#pragma pack(1)

class PLCaccessClass;
class plcsrvthread;
class plctimingthread;

#pragma pack(push, 1) // exact fit - no padding

namespace PLCaccess_ns
{
// timestamp
//typedef uint8_t tstamp_t[TSTAMP_SIZE];
typedef struct tstamp_s {
	uint8_t		year,
				month,
				day,
				hour,
				min,
				sec;
	uint16_t	msec;
	uint8_t		wday;
} tstamp__;

// structure for db logging messages
#define	DB_MESG_LEN	35
typedef char db_mesg[DB_MESG_LEN];


//data structure for data coming from plc
// structure for badge string
typedef uint8_t badge_t[BADGE_SIZE];

#define z_bit		0x0001

//bit positions
#define posto0		0x0001
#define posto1		0x0002
#define posto2		0x0004
#define posto3		0x0008
#define posto4		0x0010
#define posto5		0x0020
#define posto6		0x0040
#define posto7		0x0080

#define posto8		0x0100
#define posto9		0x0200
#define posto10		0x0400
#define posto11		0x0800
#define posto12		0x1000
#define posto13		0x2000
#define posto14		0x4000
#define posto15		0x08000

#define canc0		0xfffe
#define canc1		0xfffd
#define canc2		0xfffb
#define canc3		0xfff7
#define canc4		0xffef
#define canc5		0xffdf
#define canc6		0xffbf
#define canc7		0xff7f

//messages on door displays
#define mesg00		""
#define mesg01		"RONDA DA RIPETERE"
#define mesg02		"RONDA CONCLUSA A BUON FINE"
#define mesg03		"BADGE NON RICONOSCIUTO"
#define mesg04		"BADGE GIA' PRESENTE"
#define mesg05		"BADGE NON PRESENTE"
#define mesg06		"ATTENDI OPERAZIONE IN CORSO"
#define mesg07		"PULSANTE NON PREMUTO IN USCITA! PROSEGUIRE"
#define mesg08		""
#define mesg09		"INGRESSO NON CONSENTITO PER LISTA PRESENTI ESAURITA"
#define mesg10		"INGRESSO NON CONSENTITO PER MANCANZA TESTIMONI"
#define mesg11		"LETTORE DI BADGE NON ABILITATO"
#define mesg12		"PER USCIRE PRELEVA UN TESTIMONE/PER ENTRARE STRISCIA IL BADGE"
#define mesg13		"PER INIZIO RONDA STRISCIA IL BADGE"
#define mesg14		"MANCANO LE CONDIZIONI PER INIZIARE L'INGRESSO"
#define mesg15		""
#define mesg16		"ATTENDI INGRESSO IN CORSO/BADGE RICONOSCIUTO PRELEVA IL TESTIMONE"
#define mesg17		"ATTENDI LO SBLOCCO DELLA SERRATURA"
#define mesg18		"APRI LA PORTA ED ENTRA"
#define mesg19		"ENTRA E CHIUDI LA PORTA"
#define mesg20		"TOGLI IL TESTIMONE E CHIUDI LA PORTA"
#define mesg21		"PER FINE INGRESSO DEPOSITA IL TESTIMONE/ATTENDI INGRESSO IN CORSO"
#define mesg22		"RONDA IN CORSO INGRESSO INIBITO"
#define mesg23		"ANOMALIA IN INGRESSO ATTENDI RIPRISTINO DA SALA CONTROLLO"
#define mesg24		"ATTENDI FINE RONDA DA SALA CONTROLLO"
#define mesg25		"ATTENDI USCITA IN CORSO/PER USCIRE PREMI IL TASTO"
#define mesg26		"APRI LA PORTA ED ESCI"
#define mesg27		"ESCI E CHIUDI LA PORTA"
#define mesg28		"ATTENDI USCITA IN CORSO/DEPOSITA IL TESTIMONE"
#define mesg29		"STRISCIA IL BADGE"
#define mesg30		"RIPRELEVA IL TESTIMONE E CHIUDI LA PORTA"
#define mesg31		"ANOMALIA IN USCITA ATTENDI RIPRISTINO DA SALA CONTROLLO"

// structure for PLC db 50 with swapped timestamp
typedef struct db_50 {//304 bytes long
	u_short 	nDB;		//INT	50	indica il numero del DB
	u_short 	P_acc;		//WORD	W#16#0	 
	uint8_t		year,
 month,
 day,
 hour,
 min,
 sec,
 msec,
 wday;
	u_short 	spare0;		//WORD	W#16#0	 
	u_short 	posti;		//BOOL	FALSE	indica il posto occupato
	badge_t		badge_bo[8];	//STRING[14]	FALSE	indica il badge accettato
	badge_t		badge_inj[8];	//STRING[14]	FALSE	indica il badge accettato
	badge_t		badgeackbo;	//STRING[14]	FALSE	indica il badge riconosciuto
	badge_t		badgeackinj;	//STRING[14]	FALSE	
} db_50_;

//********************GZS**************************
//single name len
#define	PLC_NAME_LEN	16
// names
typedef unsigned char pers_name[PLC_NAME_LEN];

typedef struct intlck_dbacc{	//length 304 bytes (all)
	u_short nDB;
	u_short P_acc;
	uint8_t		year,
 month,
 day,
 hour,
 min,
 sec,
 msec,
 wday;
 u_short 	spare0;		//WORD	W#16#0	 
	//u_short 	posti0_15;		//BOOL	FALSE	indica il posto occupato
#if 0
 u_char 	posti0_7;		//BOOL	FALSE	indica il posto occupato
 u_char 	posti7_15;		//BOOL	FALSE	indica il posto occupato
#else
 u_short 	posti;		//BOOL	FALSE	indica il posto occupato
#endif
 pers_name	badge_l[18];	//STRING[14]	FALSE	indica il badge accettato
} intlck_dbacc_t;
//********************GZS**************************

// padding words
typedef u_short spare_t;

#define	NUM_DATA_BLK	158
// structure for PLC db 51: type, name, offset, description
typedef struct db_51 {//344 bytes long
	u_short 	nDB;		//0	INT	51	indica il numero del DB
	spare_t 		spare20;		 
	uint8_t		year,
 month,
 day,
 hour,
 min,
 sec,
 msec,
 wday;
	u_short 	blk[NUM_DATA_BLK];
	u_int32_t		msg_est_bo;	//328	message nr on external display boost
	u_int32_t		msg_int_bo;	//332	message nr on internal display boost
	u_int32_t		msg_est_inj;	//336	message nr on external display inj
	u_int32_t		msg_int_inj;	//340	message nr on internal display inj
	
	/* 14/05/2010 LZ
	u_int32_t		max_long_time_i_lost;	//108	maximum long term current lost
	u_int32_t		max_short_time_i_lost;	//112	maximum short term current lost
	*/
} db_51_;


// names
//typedef unsigned char pers_name[NUM_BIT];

// ronda names, controlled access names
typedef struct db_12x {//1856 bytes long
	pers_name	pn[DB_NAMES_LEN];
} db_12x_;
//data structure for commands to plc
#define CALL_RDA_BO		0x0100
#define CMD_STRT_RDA_BO		0x0200
#define CMD_END_RDA_BO		0x0400
#define CMD_STRT_RDA_INT	0x0800
#define CMD_END_RDA_INT		0x1000
#define CMD_STRT_RDA_DX		0x2000
#define CMD_END_RDA_DX		0x4000
#define CMD_STRT_RDA_SX		0x8000

#define CMD_END_RDA_SX		0x0001
#define CMD_STRT_RDA_ROOF	0x0002
#define CMD_END_RDA_ROOF	0x0004
#define RIPR_ING_BO		0x0008
#define RIPR_ING_INJ		0x0010
#define CMD_RST_TOPUP_ERR	0x0020
#define CMD_OPN_DR_BO		0x0040
#define CMD_OPN_DR_INJ		0x0080


// structure for PLC db tx: type, name, orig_type, default_val, description
typedef struct plc_tx {//48 bytes long
	u_short	nDB;			//INT	0	con 101 cancella badge con 102 invia i comandi 6/9 i cmd bool sempre eseguiti
	uint8_t		year,
 month,
 day,
 hour,
 min,
 sec,
 msec,
 wday;			//timestamp for ck alignment
	u_short spare05;		//WORD	W#16#0	
	u_short spare06;		//WORD	W#16#0	
	u_short posti_occupati;		//WORD	W#16#0	parola con il posto del nome da cancellare(posto a 1) 0000_0000_0000_0000
	u_short OPN_CMD_BST_B13_1;	//INT	0	Codice apertura Beam Stopper da sala controllo WST
	u_short CLD_CMD_BST_B13_1;	//INT	0	Codice chiusura Beam Stopper da sala controllo WST
	u_short OPN_CMD_BST_BTS1_1;	//INT	0	Codice apertura Beam Stopper da sala controllo WST
	u_short CLD_CMD_BST_BTS1_1;	//INT	0	Codice chiusura Beam Stopper da sala controllo WST
	u_short OPN_CMD_BST_BTS1_2;	//INT	0	Codice apertura Beam Stopper da sala controllo WST
	u_short CLD_CMD_BST_BTS1_2;	//INT	0	Codice chiusura Beam Stopper da sala controllo WST
	u_short OPN_CMD_BST_BTS1_3;	//INT	0	Codice apertura Beam Stopper da sala controllo WST
	u_short CLD_CMD_BST_BTS1_3;	//INT	0	Codice chiusura Beam Stopper da sala controllo WST
	u_short REINT_PASS1;		//INT	0	Codice per reintegro da passivazione
	u_short FRC_CLD_BST_BL;		//INT	0	Codice forzatura chiusura Beam Stopper linee da sala controllo WST
	u_short SET_TOP_UP;		//INT	0	Codice settaggio modo top_up da sala controllo WST
	u_short ACK_EME;		//INT	0	Codice per riconoscimento pulsante di emergenza premuto
	u_short RSET_TOP_UP;		//INT	0	 
	u_short GUN_ON_CMD;		//INT	0	Comando accensione gun
	u_short GUN_OFF_CMD;		//INT	0	Comando spegnimento gun
	u_short ronda_stat;		
} plc_tx_;

//TOPUP
typedef struct nshot {	//4
	u_char		c0,
			c1,
			c2,
			c3;
} nshot_t;
typedef struct tblk{//36
	u_char		c0[4*TOPUP_BLOCK_LEN];
} tblk_t;
typedef struct topup_dbcurr {	//50 byte
	u_short nDB;//2
	uint8_t		year,
 month,
 day,
 hour,
 min,
 sec,
 msec,
 wday;//8
	nshot nshot1;//4
	tblk 	blk;//36 ----------------------------> delete from here
/*	u_char		month2,
			year2,
			hour2,
			day2,
			sec2,
			min2,
			wday2,
  			msec2;
	nshot nshot2;
	tblk 	blk2;
	u_char		month3,
			year3,
			hour3,
			day3,
			sec3,
			min3,
			wday3,
  			msec3;
	nshot nshot3;
	tblk 	blk3;//--------------------------------< to here*/
} topup_dbcurr_t;

typedef struct topup_dblog {	//byte
	double		topup_tst;//timestamp
	u_short nshot;//4
	double 	c[9];
} topup_dblog_t;

#define CHECK_TOPUP_INJ_LEN	16
#define CHECK_TOPUP_TEST_LEN	14
#define CHECK_INJ_SR_LEN	14
#define CHECK_OPN_BST1_1_LEN	13
#define CHECK_OPN_BST1_2_LEN	13
#define CHECK_OPN_BST1_3_LEN	13
#define CHECK_GUN_TR_ABI_LEN	30
#define TOPUP_ERR_LEN		15


//TOPUP reset boosteron topup / gun trigger no abi


/**
 * Class Description:
 * manages I/O with PLC of access control
 */

/*
 *	Device States Description:
*  Tango::ON :       *
*  Tango::FAULT :    *
*  Tango::DISABLE :
 */


class PLCaccess: public TANGO_BASE_CLASS
{
public :
	//	Add your own data members here
	//-----------------------------------------
	u_char 	blk_db51_buffer[NUM_DATA_BLK*2];	//TODO: check right size of db51
	char dbtable[80];

	//	Here is the Start of the automatic code generation part
	//-------------------------------------------------------------	
/**
 *	@name attributes
 *	Attributs member data.
 */
//@{
		Tango::DevString	*attr_Badge_ack_bo_read;
		Tango::DevString	*attr_Badge_ack_inj_read;
		Tango::DevBoolean	*attr_Plc_status_read;
		Tango::DevString	*attr_Plc_timestamp_read;
		Tango::DevShort	*attr_Msg_est_bo_read;
		Tango::DevShort	*attr_Msg_int_bo_read;
		Tango::DevShort	*attr_Msg_est_inj_read;
		Tango::DevShort	*attr_Msg_int_inj_read;
		Tango::DevString	*attr_Version_read;
		Tango::DevBoolean	*attr_Gun_trigger_on_read;
		Tango::DevBoolean	*attr_Gun_trigger_interlock_abi_read;
		Tango::DevBoolean	*attr_Gun_trigger_access_abi_read;
		Tango::DevBoolean	*attr_On_topup_read;
		Tango::DevUShort	*attr_Shot_number_read;
		Tango::DevDouble	*attr_Curr_lost_short_term_read;
		Tango::DevDouble	*attr_Curr_lost_long_term_read;
		Tango::DevDouble	*attr_Max_long_time_read;
		Tango::DevDouble	*attr_Max_short_time_read;
		Tango::DevBoolean	*attr_Posto_bo_read;
		Tango::DevString	*attr_Badge_bo_read;
		Tango::DevBoolean	*attr_Posto_inj_read;
		Tango::DevString	*attr_Badge_inj_read;
		Tango::DevBoolean	*attr_Panel_bo_read;
		Tango::DevBoolean	*attr_Panel_inj_read;
		Tango::DevBoolean	*attr_Dr_in_bo_stat_read;
		Tango::DevBoolean	*attr_Dr_in_bo_proc_read;
		Tango::DevBoolean	*attr_Dr_in_inj_stat_read;
		Tango::DevBoolean	*attr_Dr_in_inj_proc_read;
		Tango::DevBoolean	*attr_Alarm_door_bo_read;
		Tango::DevBoolean	*attr_Alarm_door_inj_read;
		Tango::DevBoolean	*attr_Lock_bo_read;
		Tango::DevBoolean	*attr_Lock_inj_read;
		Tango::DevBoolean	*attr_Boost_stat_read;
		Tango::DevBoolean	*attr_Machine_stat_read;
		Tango::DevBoolean	*attr_Rf_stat_read;
		Tango::DevBoolean	*attr_Semaph_bo_read;
		Tango::DevBoolean	*attr_Semaph_inj_read;
		Tango::DevBoolean	*attr_Rf_enable_read;
		Tango::DevBoolean	*attr_Bo_top_up_read;
		Tango::DevBoolean	*attr_Abi_rf_read;
		Tango::DevBoolean	*attr_Ring_abi_read;
		Tango::DevBoolean	*attr_BstB_13_1_read;
		Tango::DevBoolean	*attr_BstBts_1_1_read;
		Tango::DevBoolean	*attr_BstBts_1_2_read;
		Tango::DevBoolean	*attr_BstBts_1_3_read;
		Tango::DevBoolean	*attr_Doors_1_read;
		Tango::DevBoolean	*attr_Doors_2_read;
		Tango::DevBoolean	*attr_Eme_buttons_read;
		Tango::DevBoolean	*attr_Lamp_read;
		Tango::DevBoolean	*attr_Gamma_monitor_read;
		Tango::DevBoolean	*attr_Keys_read;
		Tango::DevBoolean	*attr_Sir_buz_read;
		Tango::DevBoolean	*attr_Part_bstB_13_1_read;
		Tango::DevBoolean	*attr_Part_bstB_1_1_read;
		Tango::DevBoolean	*attr_Part_bstB_1_2_read;
		Tango::DevBoolean	*attr_Part_bstB_1_3_read;
		Tango::DevBoolean	*attr_Ronda_all_read;
		Tango::DevBoolean	*attr_Ronda_proc_read;
		Tango::DevBoolean	*attr_Ronda_alm_read;
		Tango::DevBoolean	*attr_Ronda_rb_read;
		Tango::DevBoolean	*attr_Ronda_rbl_read;
		Tango::DevBoolean	*attr_Ronda_barr_bst_read;
		Tango::DevBoolean	*attr_Old_Doors_1_read;
		Tango::DevBoolean	*attr_Old_Doors_2_read;
		Tango::DevBoolean	*attr_Timers_read;
		Tango::DevBoolean	*attr_Release_key_bo_read;
		Tango::DevBoolean	*attr_Release_key_inj_read;
		Tango::DevBoolean	*attr_Rf_booster_read;
		Tango::DevBoolean	*attr_BstBts_2_1_read;
		Tango::DevBoolean	*attr_Puls_rda_np_read;
		Tango::DevBoolean	*attr_Ronda_pt_read;
		Tango::DevBoolean	*attr_Doors_all_read;
		Tango::DevBoolean	*attr_Old_doors_all_read;
		Tango::DevBoolean	*attr_Ronda_roof_read;
		Tango::DevBoolean	*attr_Ronda_int_read;
		Tango::DevBoolean	*attr_Ronda_dx_read;
		Tango::DevBoolean	*attr_Ronda_sx_read;
		Tango::DevBoolean	*attr_Old_eme_buttons_read;
		Tango::DevString	*attr_Ronda_names_read;
		Tango::DevString	*attr_Ctrl_names_read;
		Tango::DevBoolean	*attr_Resettable_boards_read;
		Tango::DevBoolean	*attr_Disabled_boards_read;
		Tango::DevShort	*attr_Ronda_req_cause_read;
		Tango::DevBoolean	*attr_Ronda_bo_fail_cause_read;
		Tango::DevBoolean	*attr_Ronda_int_fail_cause_read;
		Tango::DevBoolean	*attr_Ronda_roof_fail_cause_read;
		Tango::DevBoolean	*attr_Ronda_dx_fail_cause_read;
		Tango::DevBoolean	*attr_Ronda_sx_fail_cause_read;
		Tango::DevBoolean	*attr_Memo_barr_read;
		Tango::DevBoolean	*attr_Doors_ronda_all_read;
		Tango::DevBoolean	*attr_Ronda_fail_dr_read;
		Tango::DevBoolean	*attr_Part_bstB_2_1_read;
		Tango::DevBoolean	*attr_Ramping_condition_read;
		Tango::DevDouble	*attr_Curr_shot_read;
		Tango::DevDouble	*attr_B_current_read;
		Tango::DevDouble	*attr_SR_current_read;
		Tango::DevDouble	*attr_B_SR_current_read;
		Tango::DevDouble	*attr_I0_ST_current_read;
		Tango::DevDouble	*attr_INJ_ST_current_read;
		Tango::DevDouble	*attr_ACCU_ST_current_read;
		Tango::DevDouble	*attr_LOST_ST_current_read;
		Tango::DevDouble	*attr_LOST_LT_current_read;
		Tango::DevDouble	*attr_Efficiency_read;
		Tango::DevBoolean	*attr_Check_topup_inj_read;
		Tango::DevBoolean	*attr_Check_topup_test_read;
		Tango::DevBoolean	*attr_Check_inj_sr_read;
		Tango::DevBoolean	*attr_Check_open_bst1_1_read;
		Tango::DevBoolean	*attr_Check_open_bst1_2_read;
		Tango::DevBoolean	*attr_Check_open_bst1_3_read;
		Tango::DevBoolean	*attr_Check_guntr_abi_read;
		Tango::DevBoolean	*attr_Topup_err_read;
		Tango::DevBoolean	*attr_DBState_read;
		Tango::DevString	*attr_DBError_read;
		Tango::DevLong	*attr_Min_I_TopUp_read;
		Tango::DevBoolean	*attr_Topup_GeV_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_008_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_048_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_064_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_080_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_100_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_112_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_118_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_124_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_130_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_136_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_142_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_148_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_223_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_229_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_235_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_247_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_253_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_259_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_265_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_278_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_291_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_304_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_310_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_316_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_321_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_333_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_339_read;
		Tango::DevBoolean	*attr_Booster_board_disabled_345_read;
//@}

/**
 *	@name Device properties
 *	Device properties member data.
 */
//@{
/**
 *	ip address of PLC
 */
	string	ipaddr;
/**
 *	tcp port for communication with PLC
 */
	Tango::DevUShort	plc_port;
/**
 *	max time in ms between two messages from PLC
 */
	Tango::DevUShort	timeout;
/**
 *	host, user, passw, port for db log connection
 */
	vector<string>	db_param;
/**
 *	string containing the timeout in msec of db50 and db51 data received from plc
 */
	string	hbeat;
//@}

/**@name Constructors
 * Miscellaneous constructors */
//@{
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	PLCaccess(Tango::DeviceClass *cl,string &s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	PLCaccess(Tango::DeviceClass *cl,const char *s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device name
 *	@param d	Device description.
 */
	PLCaccess(Tango::DeviceClass *cl,const char *s,const char *d);
//@}

/**@name Destructor
 * Only one desctructor is defined for this class */
//@{
/**
 * The object desctructor.
 */	
	~PLCaccess() {delete_device();};
/**
 *	will be called at device destruction or at init command.
 */
	void delete_device();
//@}

	
/**@name Miscellaneous methods */
//@{
/**
 *	Initialize the device
 */
	virtual void init_device();
/**
 *	Always executed method befor execution command method.
 */
	virtual void always_executed_hook();

//@}

/**
 * @name PLCaccess methods prototypes
 */

//@{
/**
 *	Hardware acquisition for attributes.
 */
	virtual void read_attr_hardware(vector<long> &attr_list);
/**
 *	Extract real attribute values for Badge_ack_bo acquisition result.
 */
	virtual void read_Badge_ack_bo(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Badge_ack_inj acquisition result.
 */
	virtual void read_Badge_ack_inj(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Plc_status acquisition result.
 */
	virtual void read_Plc_status(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Plc_timestamp acquisition result.
 */
	virtual void read_Plc_timestamp(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Msg_est_bo acquisition result.
 */
	virtual void read_Msg_est_bo(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Msg_int_bo acquisition result.
 */
	virtual void read_Msg_int_bo(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Msg_est_inj acquisition result.
 */
	virtual void read_Msg_est_inj(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Msg_int_inj acquisition result.
 */
	virtual void read_Msg_int_inj(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Version acquisition result.
 */
	virtual void read_Version(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Gun_trigger_on acquisition result.
 */
	virtual void read_Gun_trigger_on(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Gun_trigger_interlock_abi acquisition result.
 */
	virtual void read_Gun_trigger_interlock_abi(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Gun_trigger_access_abi acquisition result.
 */
	virtual void read_Gun_trigger_access_abi(Tango::Attribute &attr);
/**
 *	Extract real attribute values for On_topup acquisition result.
 */
	virtual void read_On_topup(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Shot_number acquisition result.
 */
	virtual void read_Shot_number(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Curr_lost_short_term acquisition result.
 */
	virtual void read_Curr_lost_short_term(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Curr_lost_long_term acquisition result.
 */
	virtual void read_Curr_lost_long_term(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Max_long_time acquisition result.
 */
	virtual void read_Max_long_time(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Max_short_time acquisition result.
 */
	virtual void read_Max_short_time(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Posto_bo acquisition result.
 */
	virtual void read_Posto_bo(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Badge_bo acquisition result.
 */
	virtual void read_Badge_bo(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Posto_inj acquisition result.
 */
	virtual void read_Posto_inj(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Badge_inj acquisition result.
 */
	virtual void read_Badge_inj(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Panel_bo acquisition result.
 */
	virtual void read_Panel_bo(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Panel_inj acquisition result.
 */
	virtual void read_Panel_inj(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Dr_in_bo_stat acquisition result.
 */
	virtual void read_Dr_in_bo_stat(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Dr_in_bo_proc acquisition result.
 */
	virtual void read_Dr_in_bo_proc(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Dr_in_inj_stat acquisition result.
 */
	virtual void read_Dr_in_inj_stat(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Dr_in_inj_proc acquisition result.
 */
	virtual void read_Dr_in_inj_proc(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Alarm_door_bo acquisition result.
 */
	virtual void read_Alarm_door_bo(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Alarm_door_inj acquisition result.
 */
	virtual void read_Alarm_door_inj(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Lock_bo acquisition result.
 */
	virtual void read_Lock_bo(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Lock_inj acquisition result.
 */
	virtual void read_Lock_inj(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Boost_stat acquisition result.
 */
	virtual void read_Boost_stat(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Machine_stat acquisition result.
 */
	virtual void read_Machine_stat(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Rf_stat acquisition result.
 */
	virtual void read_Rf_stat(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Semaph_bo acquisition result.
 */
	virtual void read_Semaph_bo(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Semaph_inj acquisition result.
 */
	virtual void read_Semaph_inj(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Rf_enable acquisition result.
 */
	virtual void read_Rf_enable(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Bo_top_up acquisition result.
 */
	virtual void read_Bo_top_up(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Abi_rf acquisition result.
 */
	virtual void read_Abi_rf(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ring_abi acquisition result.
 */
	virtual void read_Ring_abi(Tango::Attribute &attr);
/**
 *	Extract real attribute values for BstB_13_1 acquisition result.
 */
	virtual void read_BstB_13_1(Tango::Attribute &attr);
/**
 *	Extract real attribute values for BstBts_1_1 acquisition result.
 */
	virtual void read_BstBts_1_1(Tango::Attribute &attr);
/**
 *	Extract real attribute values for BstBts_1_2 acquisition result.
 */
	virtual void read_BstBts_1_2(Tango::Attribute &attr);
/**
 *	Extract real attribute values for BstBts_1_3 acquisition result.
 */
	virtual void read_BstBts_1_3(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Doors_1 acquisition result.
 */
	virtual void read_Doors_1(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Doors_2 acquisition result.
 */
	virtual void read_Doors_2(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Eme_buttons acquisition result.
 */
	virtual void read_Eme_buttons(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Lamp acquisition result.
 */
	virtual void read_Lamp(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Gamma_monitor acquisition result.
 */
	virtual void read_Gamma_monitor(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Keys acquisition result.
 */
	virtual void read_Keys(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Sir_buz acquisition result.
 */
	virtual void read_Sir_buz(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Part_bstB_13_1 acquisition result.
 */
	virtual void read_Part_bstB_13_1(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Part_bstB_1_1 acquisition result.
 */
	virtual void read_Part_bstB_1_1(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Part_bstB_1_2 acquisition result.
 */
	virtual void read_Part_bstB_1_2(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Part_bstB_1_3 acquisition result.
 */
	virtual void read_Part_bstB_1_3(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_all acquisition result.
 */
	virtual void read_Ronda_all(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_proc acquisition result.
 */
	virtual void read_Ronda_proc(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_alm acquisition result.
 */
	virtual void read_Ronda_alm(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_rb acquisition result.
 */
	virtual void read_Ronda_rb(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_rbl acquisition result.
 */
	virtual void read_Ronda_rbl(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_barr_bst acquisition result.
 */
	virtual void read_Ronda_barr_bst(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Old_Doors_1 acquisition result.
 */
	virtual void read_Old_Doors_1(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Old_Doors_2 acquisition result.
 */
	virtual void read_Old_Doors_2(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Timers acquisition result.
 */
	virtual void read_Timers(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Release_key_bo acquisition result.
 */
	virtual void read_Release_key_bo(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Release_key_inj acquisition result.
 */
	virtual void read_Release_key_inj(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Rf_booster acquisition result.
 */
	virtual void read_Rf_booster(Tango::Attribute &attr);
/**
 *	Extract real attribute values for BstBts_2_1 acquisition result.
 */
	virtual void read_BstBts_2_1(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Puls_rda_np acquisition result.
 */
	virtual void read_Puls_rda_np(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_pt acquisition result.
 */
	virtual void read_Ronda_pt(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Doors_all acquisition result.
 */
	virtual void read_Doors_all(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Old_doors_all acquisition result.
 */
	virtual void read_Old_doors_all(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_roof acquisition result.
 */
	virtual void read_Ronda_roof(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_int acquisition result.
 */
	virtual void read_Ronda_int(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_dx acquisition result.
 */
	virtual void read_Ronda_dx(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_sx acquisition result.
 */
	virtual void read_Ronda_sx(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Old_eme_buttons acquisition result.
 */
	virtual void read_Old_eme_buttons(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_names acquisition result.
 */
	virtual void read_Ronda_names(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ctrl_names acquisition result.
 */
	virtual void read_Ctrl_names(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Resettable_boards acquisition result.
 */
	virtual void read_Resettable_boards(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Disabled_boards acquisition result.
 */
	virtual void read_Disabled_boards(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_req_cause acquisition result.
 */
	virtual void read_Ronda_req_cause(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_bo_fail_cause acquisition result.
 */
	virtual void read_Ronda_bo_fail_cause(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_int_fail_cause acquisition result.
 */
	virtual void read_Ronda_int_fail_cause(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_roof_fail_cause acquisition result.
 */
	virtual void read_Ronda_roof_fail_cause(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_dx_fail_cause acquisition result.
 */
	virtual void read_Ronda_dx_fail_cause(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_sx_fail_cause acquisition result.
 */
	virtual void read_Ronda_sx_fail_cause(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Memo_barr acquisition result.
 */
	virtual void read_Memo_barr(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Doors_ronda_all acquisition result.
 */
	virtual void read_Doors_ronda_all(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ronda_fail_dr acquisition result.
 */
	virtual void read_Ronda_fail_dr(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Part_bstB_2_1 acquisition result.
 */
	virtual void read_Part_bstB_2_1(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Ramping_condition acquisition result.
 */
	virtual void read_Ramping_condition(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Curr_shot acquisition result.
 */
	virtual void read_Curr_shot(Tango::Attribute &attr);
/**
 *	Extract real attribute values for B_current acquisition result.
 */
	virtual void read_B_current(Tango::Attribute &attr);
/**
 *	Extract real attribute values for SR_current acquisition result.
 */
	virtual void read_SR_current(Tango::Attribute &attr);
/**
 *	Extract real attribute values for B_SR_current acquisition result.
 */
	virtual void read_B_SR_current(Tango::Attribute &attr);
/**
 *	Extract real attribute values for I0_ST_current acquisition result.
 */
	virtual void read_I0_ST_current(Tango::Attribute &attr);
/**
 *	Extract real attribute values for INJ_ST_current acquisition result.
 */
	virtual void read_INJ_ST_current(Tango::Attribute &attr);
/**
 *	Extract real attribute values for ACCU_ST_current acquisition result.
 */
	virtual void read_ACCU_ST_current(Tango::Attribute &attr);
/**
 *	Extract real attribute values for LOST_ST_current acquisition result.
 */
	virtual void read_LOST_ST_current(Tango::Attribute &attr);
/**
 *	Extract real attribute values for LOST_LT_current acquisition result.
 */
	virtual void read_LOST_LT_current(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Efficiency acquisition result.
 */
	virtual void read_Efficiency(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Check_topup_inj acquisition result.
 */
	virtual void read_Check_topup_inj(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Check_topup_test acquisition result.
 */
	virtual void read_Check_topup_test(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Check_inj_sr acquisition result.
 */
	virtual void read_Check_inj_sr(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Check_open_bst1_1 acquisition result.
 */
	virtual void read_Check_open_bst1_1(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Check_open_bst1_2 acquisition result.
 */
	virtual void read_Check_open_bst1_2(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Check_open_bst1_3 acquisition result.
 */
	virtual void read_Check_open_bst1_3(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Check_guntr_abi acquisition result.
 */
	virtual void read_Check_guntr_abi(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Topup_err acquisition result.
 */
	virtual void read_Topup_err(Tango::Attribute &attr);
/**
 *	Extract real attribute values for DBState acquisition result.
	*/
	virtual void read_DBState(Tango::Attribute &attr);
/**
 *	Extract real attribute values for DBError acquisition result.
 */
	virtual void read_DBError(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Min_I_TopUp acquisition result.
 */
	virtual void read_Min_I_TopUp(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Topup_GeV acquisition result.
 */
	virtual void read_Topup_GeV(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Booster_board_disabled_xxx acquisition result.
 */
	virtual void read_Booster_board_disabled_008(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_048(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_064(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_080(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_100(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_112(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_118(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_124(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_130(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_136(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_142(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_148(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_223(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_229(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_235(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_247(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_253(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_259(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_265(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_278(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_291(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_304(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_310(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_316(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_321(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_333(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_339(Tango::Attribute &attr);
	virtual void read_Booster_board_disabled_345(Tango::Attribute &attr);
/**
 *	Read/Write allowed for Badge_ack_bo attribute.
 */
	virtual bool is_Badge_ack_bo_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Badge_ack_inj attribute.
 */
	virtual bool is_Badge_ack_inj_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Plc_status attribute.
 */
	virtual bool is_Plc_status_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Plc_timestamp attribute.
 */
	virtual bool is_Plc_timestamp_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Msg_est_bo attribute.
 */
	virtual bool is_Msg_est_bo_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Msg_int_bo attribute.
 */
	virtual bool is_Msg_int_bo_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Msg_est_inj attribute.
 */
	virtual bool is_Msg_est_inj_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Msg_int_inj attribute.
 */
	virtual bool is_Msg_int_inj_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Version attribute.
 */
	virtual bool is_Version_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Gun_trigger_on attribute.
 */
	virtual bool is_Gun_trigger_on_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Gun_trigger_interlock_abi attribute.
 */
	virtual bool is_Gun_trigger_interlock_abi_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Gun_trigger_access_abi attribute.
 */
	virtual bool is_Gun_trigger_access_abi_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for On_topup attribute.
 */
	virtual bool is_On_topup_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Shot_number attribute.
 */
	virtual bool is_Shot_number_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Curr_lost_short_term attribute.
 */
	virtual bool is_Curr_lost_short_term_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Curr_lost_long_term attribute.
 */
	virtual bool is_Curr_lost_long_term_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Max_long_time attribute.
 */
	virtual bool is_Max_long_time_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Max_short_time attribute.
 */
	virtual bool is_Max_short_time_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Posto_bo attribute.
 */
	virtual bool is_Posto_bo_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Badge_bo attribute.
 */
	virtual bool is_Badge_bo_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Posto_inj attribute.
 */
	virtual bool is_Posto_inj_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Badge_inj attribute.
 */
	virtual bool is_Badge_inj_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Panel_bo attribute.
 */
	virtual bool is_Panel_bo_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Panel_inj attribute.
 */
	virtual bool is_Panel_inj_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Dr_in_bo_stat attribute.
 */
	virtual bool is_Dr_in_bo_stat_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Dr_in_bo_proc attribute.
 */
	virtual bool is_Dr_in_bo_proc_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Dr_in_inj_stat attribute.
 */
	virtual bool is_Dr_in_inj_stat_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Dr_in_inj_proc attribute.
 */
	virtual bool is_Dr_in_inj_proc_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Alarm_door_bo attribute.
 */
	virtual bool is_Alarm_door_bo_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Alarm_door_inj attribute.
 */
	virtual bool is_Alarm_door_inj_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Lock_bo attribute.
 */
	virtual bool is_Lock_bo_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Lock_inj attribute.
 */
	virtual bool is_Lock_inj_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Boost_stat attribute.
 */
	virtual bool is_Boost_stat_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Machine_stat attribute.
 */
	virtual bool is_Machine_stat_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Rf_stat attribute.
 */
	virtual bool is_Rf_stat_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Semaph_bo attribute.
 */
	virtual bool is_Semaph_bo_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Semaph_inj attribute.
 */
	virtual bool is_Semaph_inj_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Rf_enable attribute.
 */
	virtual bool is_Rf_enable_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Bo_top_up attribute.
 */
	virtual bool is_Bo_top_up_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Abi_rf attribute.
 */
	virtual bool is_Abi_rf_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ring_abi attribute.
 */
	virtual bool is_Ring_abi_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for BstB_13_1 attribute.
 */
	virtual bool is_BstB_13_1_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for BstBts_1_1 attribute.
 */
	virtual bool is_BstBts_1_1_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for BstBts_1_2 attribute.
 */
	virtual bool is_BstBts_1_2_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for BstBts_1_3 attribute.
 */
	virtual bool is_BstBts_1_3_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Doors_1 attribute.
 */
	virtual bool is_Doors_1_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Doors_2 attribute.
 */
	virtual bool is_Doors_2_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Eme_buttons attribute.
 */
	virtual bool is_Eme_buttons_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Lamp attribute.
 */
	virtual bool is_Lamp_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Gamma_monitor attribute.
 */
	virtual bool is_Gamma_monitor_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Keys attribute.
 */
	virtual bool is_Keys_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Sir_buz attribute.
 */
	virtual bool is_Sir_buz_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Part_bstB_13_1 attribute.
 */
	virtual bool is_Part_bstB_13_1_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Part_bstB_1_1 attribute.
 */
	virtual bool is_Part_bstB_1_1_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Part_bstB_1_2 attribute.
 */
	virtual bool is_Part_bstB_1_2_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Part_bstB_1_3 attribute.
 */
	virtual bool is_Part_bstB_1_3_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_all attribute.
 */
	virtual bool is_Ronda_all_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_proc attribute.
 */
	virtual bool is_Ronda_proc_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_alm attribute.
 */
	virtual bool is_Ronda_alm_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_rb attribute.
 */
	virtual bool is_Ronda_rb_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_rbl attribute.
 */
	virtual bool is_Ronda_rbl_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_barr_bst attribute.
 */
	virtual bool is_Ronda_barr_bst_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Old_Doors_1 attribute.
 */
	virtual bool is_Old_Doors_1_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Old_Doors_2 attribute.
 */
	virtual bool is_Old_Doors_2_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Timers attribute.
 */
	virtual bool is_Timers_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Release_key_bo attribute.
 */
	virtual bool is_Release_key_bo_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Release_key_inj attribute.
 */
	virtual bool is_Release_key_inj_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Rf_booster attribute.
 */
	virtual bool is_Rf_booster_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for BstBts_2_1 attribute.
 */
	virtual bool is_BstBts_2_1_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Puls_rda_np attribute.
 */
	virtual bool is_Puls_rda_np_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_pt attribute.
 */
	virtual bool is_Ronda_pt_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Doors_all attribute.
 */
	virtual bool is_Doors_all_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Old_doors_all attribute.
 */
	virtual bool is_Old_doors_all_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_roof attribute.
 */
	virtual bool is_Ronda_roof_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_int attribute.
 */
	virtual bool is_Ronda_int_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_dx attribute.
 */
	virtual bool is_Ronda_dx_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_sx attribute.
 */
	virtual bool is_Ronda_sx_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Old_eme_buttons attribute.
 */
	virtual bool is_Old_eme_buttons_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_names attribute.
 */
	virtual bool is_Ronda_names_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ctrl_names attribute.
 */
	virtual bool is_Ctrl_names_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Resettable_boards attribute.
 */
	virtual bool is_Resettable_boards_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Disabled_boards attribute.
 */
	virtual bool is_Disabled_boards_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_req_cause attribute.
 */
	virtual bool is_Ronda_req_cause_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_bo_fail_cause attribute.
 */
	virtual bool is_Ronda_bo_fail_cause_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_int_fail_cause attribute.
 */
	virtual bool is_Ronda_int_fail_cause_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_roof_fail_cause attribute.
 */
	virtual bool is_Ronda_roof_fail_cause_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_dx_fail_cause attribute.
 */
	virtual bool is_Ronda_dx_fail_cause_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_sx_fail_cause attribute.
 */
	virtual bool is_Ronda_sx_fail_cause_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Memo_barr attribute.
 */
	virtual bool is_Memo_barr_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Doors_ronda_all attribute.
 */
	virtual bool is_Doors_ronda_all_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ronda_fail_dr attribute.
 */
	virtual bool is_Ronda_fail_dr_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Part_bstB_2_1 attribute.
 */
	virtual bool is_Part_bstB_2_1_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Ramping_condition attribute.
 */
	virtual bool is_Ramping_condition_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Curr_shot attribute.
 */
	virtual bool is_Curr_shot_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for B_current attribute.
 */
	virtual bool is_B_current_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for SR_current attribute.
 */
	virtual bool is_SR_current_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for B_SR_current attribute.
 */
	virtual bool is_B_SR_current_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for I0_ST_current attribute.
 */
	virtual bool is_I0_ST_current_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for INJ_ST_current attribute.
 */
	virtual bool is_INJ_ST_current_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for ACCU_ST_current attribute.
 */
	virtual bool is_ACCU_ST_current_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for LOST_ST_current attribute.
 */
	virtual bool is_LOST_ST_current_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for LOST_LT_current attribute.
 */
	virtual bool is_LOST_LT_current_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Efficiency attribute.
 */
	virtual bool is_Efficiency_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Check_topup_inj attribute.
 */
	virtual bool is_Check_topup_inj_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Check_topup_test attribute.
 */
	virtual bool is_Check_topup_test_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Check_inj_sr attribute.
 */
	virtual bool is_Check_inj_sr_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Check_open_bst1_1 attribute.
 */
	virtual bool is_Check_open_bst1_1_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Check_open_bst1_2 attribute.
 */
	virtual bool is_Check_open_bst1_2_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Check_open_bst1_3 attribute.
 */
	virtual bool is_Check_open_bst1_3_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Check_guntr_abi attribute.
 */
	virtual bool is_Check_guntr_abi_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Topup_err attribute.
 */
	virtual bool is_Topup_err_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for DBState attribute.
 */
	virtual bool is_DBState_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for DBError attribute.
 */
	virtual bool is_DBError_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Min_I_TopUp attribute.
 */
	virtual bool is_Min_I_TopUp_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Topup_GeV attribute.
 */
	virtual bool is_Topup_GeV_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Booster_board_disabled_xxx attribute.
 */
	virtual bool is_Booster_board_disabled_008_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_048_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_064_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_080_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_100_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_112_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_118_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_124_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_130_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_136_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_142_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_148_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_223_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_229_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_235_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_247_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_253_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_259_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_265_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_278_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_291_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_304_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_310_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_316_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_321_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_333_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_339_allowed(Tango::AttReqType type);
	virtual bool is_Booster_board_disabled_345_allowed(Tango::AttReqType type);
/**
 *	Execution allowed for Open_bst_b13_1 command.
 */
	virtual bool is_Open_bst_b13_1_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Close_bst_b13_1 command.
 */
	virtual bool is_Close_bst_b13_1_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Open_bst_bts1_1 command.
 */
	virtual bool is_Open_bst_bts1_1_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Close_bst_bts1_1 command.
 */
	virtual bool is_Close_bst_bts1_1_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Open_bst_bts1_2 command.
 */
	virtual bool is_Open_bst_bts1_2_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Close_bst_bts1_2 command.
 */
	virtual bool is_Close_bst_bts1_2_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Open_bst_bts1_3 command.
 */
	virtual bool is_Open_bst_bts1_3_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Close_bst_bts1_3 command.
 */
	virtual bool is_Close_bst_bts1_3_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Del_posto_bo command.
 */
	virtual bool is_Del_posto_bo_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Reint_pasv command.
 */
	virtual bool is_Reint_pasv_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Close_bst_bl command.
 */
	virtual bool is_Close_bst_bl_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Set_top_up command.
 */
	virtual bool is_Set_top_up_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Ack_eme command.
 */
	virtual bool is_Ack_eme_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Call_ronda_bo command.
 */
	virtual bool is_Call_ronda_bo_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Start_ronda_bo command.
 */
	virtual bool is_Start_ronda_bo_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for End_ronda_bo command.
 */
	virtual bool is_End_ronda_bo_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Start_ronda_int command.
 */
	virtual bool is_Start_ronda_int_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for End_ronda_int command.
 */
	virtual bool is_End_ronda_int_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Start_ronda_dx command.
 */
	virtual bool is_Start_ronda_dx_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for End_ronda_dx command.
 */
	virtual bool is_End_ronda_dx_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Start_ronda_sx command.
 */
	virtual bool is_Start_ronda_sx_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for End_ronda_sx command.
 */
	virtual bool is_End_ronda_sx_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Start_ronda_roof command.
 */
	virtual bool is_Start_ronda_roof_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for End_ronda_roof command.
 */
	virtual bool is_End_ronda_roof_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Ripr_in_boost command.
 */
	virtual bool is_Ripr_in_boost_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Ripr_in_inj command.
 */
	virtual bool is_Ripr_in_inj_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Open_door_bo command.
 */
	virtual bool is_Open_door_bo_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Open_door_inj command.
 */
	virtual bool is_Open_door_inj_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Del_posto_inj command.
 */
	virtual bool is_Del_posto_inj_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Confirm_open_13_1 command.
 */
	virtual bool is_Confirm_open_13_1_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Confirm_close_13_1 command.
 */
	virtual bool is_Confirm_close_13_1_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Confirm_open_1_1 command.
 */
	virtual bool is_Confirm_open_1_1_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Confirm_close_1_1 command.
 */
	virtual bool is_Confirm_close_1_1_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Confirm_open_1_2 command.
 */
	virtual bool is_Confirm_open_1_2_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Confirm_close_1_2 command.
 */
	virtual bool is_Confirm_close_1_2_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Confirm_open_1_3 command.
 */
	virtual bool is_Confirm_open_1_3_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Confirm_close_1_3 command.
 */
	virtual bool is_Confirm_close_1_3_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Confirm_reint_pasv command.
 */
	virtual bool is_Confirm_reint_pasv_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Confirm_close_bst_bl command.
 */
	virtual bool is_Confirm_close_bst_bl_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Confirm_set_topup command.
 */
	virtual bool is_Confirm_set_topup_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Confirm_ack_eme command.
 */
	virtual bool is_Confirm_ack_eme_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Echo_db50 command.
 */
	virtual bool is_Echo_db50_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Echo_db51 command.
 */
	virtual bool is_Echo_db51_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Echo_db121 command.
 */
	virtual bool is_Echo_db121_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Echo_db122 command.
 */
	virtual bool is_Echo_db122_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Gun_on command.
 */
	virtual bool is_Gun_on_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Gun_off command.
 */
	virtual bool is_Gun_off_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Syncronize command.
 */
	virtual bool is_Syncronize_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Rst_top_up command.
 */
	virtual bool is_Rst_top_up_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Err_topup_rst command.
 */
	virtual bool is_Err_topup_rst_allowed(const CORBA::Any &any);
/**
 * 
 *	@exception DevFailed
 */
	void	open_bst_b13_1();
/**
 * 
 *	@exception DevFailed
 */
	void	close_bst_b13_1();
/**
 * 
 *	@exception DevFailed
 */
	void	open_bst_bts1_1();
/**
 * 
 *	@exception DevFailed
 */
	void	close_bst_bts1_1();
/**
 * 
 *	@exception DevFailed
 */
	void	open_bst_bts1_2();
/**
 * 
 *	@exception DevFailed
 */
	void	close_bst_bts1_2();
/**
 * 
 *	@exception DevFailed
 */
	void	open_bst_bts1_3();
/**
 * 
 *	@exception DevFailed
 */
	void	close_bst_bts1_3();
/**
 * Deletes incorrect access of booster door corresponding to argin place (from 0 to 7)
 *	@param	argin	places of name to be deleted
 *	@exception DevFailed
 */
	void	del_posto_bo(Tango::DevUShort);
/**
 * command for activating PLC boards after an error or a breakdown
 *	@exception DevFailed
 */
	void	reint_pasv();
/**
 * 
 *	@exception DevFailed
 */
	void	close_bst_bl();
/**
 * Switches to TOP-UP mode
 *	@exception DevFailed
 */
	void	set_top_up();
/**
 * Acknowledge for emergency button pressed
 *	@exception DevFailed
 */
	void	ack_eme();
/**
 * 
 *	@exception DevFailed
 */
	void	call_ronda_bo();
/**
 * 
 *	@exception DevFailed
 */
	void	start_ronda_bo();
/**
 * 
 *	@exception DevFailed
 */
	void	end_ronda_bo();
/**
 * 
 *	@exception DevFailed
 */
	void	start_ronda_int();
/**
 * 
 *	@exception DevFailed
 */
	void	end_ronda_int();
/**
 * 
 *	@exception DevFailed
 */
	void	start_ronda_dx();
/**
 * 
 *	@exception DevFailed
 */
	void	end_ronda_dx();
/**
 * 
 *	@exception DevFailed
 */
	void	start_ronda_sx();
/**
 * 
 *	@exception DevFailed
 */
	void	end_ronda_sx();
/**
 * 
 *	@exception DevFailed
 */
	void	start_ronda_roof();
/**
 * 
 *	@exception DevFailed
 */
	void	end_ronda_roof();
/**
 * Reset inputs of booster door after access failed
 *	@exception DevFailed
 */
	void	ripr_in_boost();
/**
 * Reset inputs of injector door after access failed
 *	@exception DevFailed
 */
	void	ripr_in_inj();
/**
 * Open booster's door
 *	@exception DevFailed
 */
	void	open_door_bo();
/**
 * Open injector's door
 *	@exception DevFailed
 */
	void	open_door_inj();
/**
 * Deletes incorrect access of injector door corresponding to argin place (from 0 to 7)
 *	@param	argin	place of name to be deleted
 *	@exception DevFailed
 */
	void	del_posto_inj(Tango::DevUShort);
/**
 * 
 *	@exception DevFailed
 */
	void	confirm_open_13_1();
/**
 * 
 *	@exception DevFailed
 */
	void	confirm_close_13_1();
/**
 * 
 *	@exception DevFailed
 */
	void	confirm_open_1_1();
/**
 * 
 *	@exception DevFailed
 */
	void	confirm_close_1_1();
/**
 * 
 *	@exception DevFailed
 */
	void	confirm_open_1_2();
/**
 * 
 *	@exception DevFailed
 */
	void	confirm_close_1_2();
/**
 * 
 *	@exception DevFailed
 */
	void	confirm_open_1_3();
/**
 * 
 *	@exception DevFailed
 */
	void	confirm_close_1_3();
/**
 * 
 *	@exception DevFailed
 */
	void	confirm_reint_pasv();
/**
 * 
 *	@exception DevFailed
 */
	void	confirm_close_bst_bl();
/**
 * 
 *	@exception DevFailed
 */
	void	confirm_set_topup();
/**
 * 
 *	@exception DevFailed
 */
	void	confirm_ack_eme();
/**
 * request from plc a copy of db50, the result is mapped on the attributes
 *	@return	success of the request
 *	@exception DevFailed
 */
	Tango::DevString	echo_db50();
/**
 * requests to plc a copy of db51, the result is stored inside the attributes
 *	@return	success of request
 *	@exception DevFailed
 */
	Tango::DevString	echo_db51();
/**
 * Request the list of ronda enabled people names
 *	@return	result
 *	@exception DevFailed
 */
	Tango::DevVarStringArray	*echo_db121();
/**
 * Request the list of controlled access people names
 *	@return	result
 *	@exception DevFailed
 */
	Tango::DevVarStringArray	*echo_db122();
/**
 * 
 *	@exception DevFailed
 */
	void	gun_on();
/**
 * 
 *	@exception DevFailed
 */
	void	gun_off();
/**
 * 
 *	@exception DevFailed
 */
	void	syncronize();
/**
 * Reset TOP-UP mode
 *	@exception DevFailed
 */
	void	rst_top_up();
/**
 * Reset topup errors
 *	@exception DevFailed
 */
	void	err_topup_rst();

/**
 *	Read the device properties from database
 */
	 void get_device_property();
//@}

	//	Here is the end of the automatic code generation part
	//-------------------------------------------------------------	
	
	omni_mutex *mutex;
	plcsrvthread *srvloop;		//read from socket thread
	plctimingthread *timeloop;	//manages the read timeout
	string ip_host;		// ip host name for tcp/ip 
	int new_sd, sock;	// socket for data exchange
	//int fdlog;		//logfile
	int abortflag;
	int h_timeout[2];		//heartbeat of db50 and db51
//	tstamp_t old_timestamp;
	const char* logfile2;	//name of the logfile
	int echoreq;
	string srv_host;
	//void tofile(int btype, int len);//logging of the raw message

	deque<db_50> mesg_list_50;
	deque<db_51> mesg_list_51;
	
	MYSQL dbp;
	Tango::DevString sw_version;
	
	db_50 message;
	db_51 message2;
	plc_tx cmd_msg;
	db_12x ronda_names;
	db_12x ctrl_names;


	tstamp_s tst50;
	tstamp_s tst51;
	tstamp_s old_tst50;
	tstamp_s old_tst51;
	db_51 buf_db_51;
	
	Tango::DevBoolean rondanames_ok;//ronda allowed people successful read
	Tango::DevBoolean ctrlnames_ok;//access allowed people successful read

	//data for attributes of db_50
	Tango::DevBoolean postobo[NUM_POSTO];
	Tango::DevString badgebo[NUM_POSTO];
	Tango::DevString badgeinj[NUM_POSTO];
	Tango::DevBoolean postoinj[NUM_POSTO];
	Tango::DevString badgeackbo;
	Tango::DevString badgeackinj;
	
	//data for attributes of db_51
	//Tango::DevBoolean panelbo[NUM_POSTO];
	Tango::DevBoolean panelbo[2*NUM_POSTO];
	//Tango::DevBoolean panelinj[NUM_POSTO];
	Tango::DevBoolean panelinj[2*NUM_POSTO];
	Tango::DevBoolean dr_in_bo_stat[NUM_BIT];
	Tango::DevBoolean dr_in_bo_proc[NUM_BIT];
	Tango::DevBoolean dr_in_inj_stat[NUM_BIT];
	Tango::DevBoolean dr_in_inj_proc[NUM_BIT];
	Tango::DevBoolean alarm_door_bo[NUM_BIT];
	Tango::DevBoolean alarm_door_inj[NUM_BIT];
	Tango::DevBoolean lock_bo[NUM_BIT];
	Tango::DevBoolean lock_inj[NUM_BIT];
	Tango::DevBoolean boost_stat[NUM_BIT];
	Tango::DevBoolean machine_stat[NUM_BIT];
	Tango::DevBoolean rf_stat[NUM_BIT];
	Tango::DevBoolean semaphbo[NUM_POSTO];
	Tango::DevBoolean semaphinj[NUM_POSTO];
	Tango::DevBoolean rfenable[NUM_POSTO+1];
	//Tango::DevBoolean botopup[2];
	Tango::DevBoolean botopup[4];
	Tango::DevBoolean abirf[2];
	Tango::DevBoolean ringabi[7];
	Tango::DevBoolean bstb_13_1[NUM_POSTO];
	Tango::DevBoolean bstbts_1_1[NUM_POSTO];
	Tango::DevBoolean bstbts_1_2[NUM_POSTO];
	Tango::DevBoolean bstbts_1_3[NUM_POSTO];
	Tango::DevBoolean doors_1[NUM_BIT];
	Tango::DevBoolean doors_2[NUM_BIT];
	Tango::DevBoolean eme_buttons[NUM_BIT];
	Tango::DevBoolean lamp[NUM_BIT];
	Tango::DevBoolean gamma_monitor[3];
	Tango::DevBoolean keys[3];
	Tango::DevBoolean sir_buz[NUM_POSTO];
	Tango::DevBoolean part_bstB_13_1[6];
	Tango::DevBoolean part_bstB_1_1[6];
	Tango::DevBoolean part_bstB_1_2[6];
	Tango::DevBoolean part_bstB_1_3[6];
	Tango::DevBoolean part_bstB_2_1[6];
	Tango::DevBoolean ronda_all[NUM_BIT];
	Tango::DevBoolean ronda_proc[NUM_BIT];
	Tango::DevBoolean ronda_alm[NUM_POSTO];
	Tango::DevBoolean ronda_rb[NUM_BIT];
	Tango::DevBoolean ronda_rbl[NUM_BIT];
	Tango::DevBoolean ronda_barr_bst[NUM_BIT];
	Tango::DevBoolean ronda_roof[8];
	Tango::DevBoolean ronda_int[8];
	Tango::DevBoolean ronda_dx[8];
	Tango::DevBoolean ronda_sx[8];
	Tango::DevBoolean timers[NUM_POSTO];
	Tango::DevShort msg_ext_bo;
	Tango::DevShort msg_int_bo;
	Tango::DevShort msg_ext_inj;
	Tango::DevShort msg_int_inj;
	Tango::DevBoolean release_key_bo[NUM_POSTO];
	Tango::DevBoolean release_key_inj[NUM_POSTO];
	Tango::DevBoolean rf_booster[NUM_POSTO];
	Tango::DevBoolean bstbts_2_1[8];
	Tango::DevBoolean puls_rda_np[NUM_POSTO];
	Tango::DevBoolean ronda_pt[4];
	Tango::DevBoolean resettable_boards[28];
	Tango::DevBoolean disabled_boards[28];
	
	Tango::DevBoolean doors_all[24];
	Tango::DevBoolean old_doors_all[24];
	
	Tango::DevString *ctrlacc_names;
	Tango::DevString *rda_names;

	Tango::DevBoolean plc_status;
	Tango::DevString plc_timestamp;

	Tango::DevBoolean old_doors_1[NUM_BIT];
	Tango::DevBoolean old_doors_2[NUM_BIT];
	Tango::DevBoolean old_eme_buttons[NUM_BIT];

	Tango::DevShort ronda_req_cause[5];
	Tango::DevBoolean ronda_bo_fail_cause[21];
	Tango::DevBoolean ronda_int_fail_cause[8];
	Tango::DevBoolean ronda_roof_fail_cause[7];
	Tango::DevBoolean ronda_dx_fail_cause[9];
	Tango::DevBoolean ronda_sx_fail_cause[12];

	Tango::DevBoolean memo_barr[4];
	Tango::DevBoolean doors_ronda_all[24];
	Tango::DevBoolean ronda_fail_dr[32];
	Tango::DevBoolean memo_err_inout[2];

	bool ronda_req_resettable[5];	
	Tango::DevBoolean gun_trigger_on;
	Tango::DevBoolean gun_trigger_interlock_abi;
	Tango::DevBoolean gun_trigger_access_abi;

	Tango::DevBoolean ramping_conditions[4];
//TOPUP
	topup_dbcurr message_curr;	
	Tango::DevBoolean	msg_all[DB181_BIT_LEN];
	Tango::DevDouble	c_meas[TOPUP_BLOCK_LEN];
	Tango::DevDouble	curr_lost_short_term;
	Tango::DevDouble	curr_lost_long_term;
	Tango::DevBoolean	cmd_on_topup;
	Tango::DevBoolean	cmd_rst_err;
	double timestamp_topup;
	Tango::DevUShort	shot_number;
	int topup_sem;
	Tango::DevBoolean ontopup;  // true during injection 	
	Tango::DevDouble	c0[CURR_BUF_XLEN];
	Tango::DevDouble	c1[CURR_BUF_XLEN];
	Tango::DevDouble	c2[CURR_BUF_XLEN];
	Tango::DevDouble	c3[CURR_BUF_XLEN];
	Tango::DevDouble	c4[CURR_BUF_XLEN];
	Tango::DevDouble	c5[CURR_BUF_XLEN];
	Tango::DevDouble	c6[CURR_BUF_XLEN];
	Tango::DevDouble	c7[CURR_BUF_XLEN];
	Tango::DevDouble	r8[CURR_BUF_XLEN];
	struct timeval now;
	Tango::DevDouble	d0[CURR_BUF_XLEN];
	topup_dblog log_db_80;
	deque<topup_dblog> mesg_list_80;
	Tango::DevBoolean	check_topup_inj[CHECK_TOPUP_INJ_LEN];
	Tango::DevBoolean	check_topup_test[CHECK_TOPUP_TEST_LEN];
	Tango::DevBoolean	check_inj_sr[CHECK_INJ_SR_LEN];
	Tango::DevBoolean	check_open_bst1_1[CHECK_OPN_BST1_1_LEN];
	Tango::DevBoolean	check_open_bst1_2[CHECK_OPN_BST1_2_LEN];
	Tango::DevBoolean	check_open_bst1_3[CHECK_OPN_BST1_3_LEN];
	Tango::DevBoolean	check_guntr_abi[CHECK_GUN_TR_ABI_LEN];
	Tango::DevBoolean	topup_err[TOPUP_ERR_LEN];

//********************GZS**************************
	//to save previous people
	pers_name	blk_db50_buffer[18];

	int32_t calc_timeval(uint8_t year,uint8_t month,uint8_t day,uint8_t hour,uint8_t min,uint8_t sec,uint8_t wday);
	int32_t calc_time_ms(uint8_t,uint8_t);
	char db_error[2*DB_QUERY_LEN];
//********************GZS**************************
//TOPUP
protected :	
	//	Add your own data members here
	//-----------------------------------------
	void send_frame (unsigned char *frame, short frame_length);
	void send_cmd(void);
};
#pragma pack(pop) //back to whatever the previous packing mode was

}	// namespace_ns

#endif	// _PLCACCESS_H
